@@ -1,8 +1,9 @@
+#include <cstdint>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fstream>
-#include <ext/hash_map>
+#include <unordered_map>
 
 #include "HashFunc.h"
 #include "rdbinterval.h"
@@ -16,12 +17,12 @@
 using namespace std;
 using namespace rdb;
 
-typedef hash_map<pair<size_t, size_t>, double> Contacts;
-typedef pair<size_t, size_t> Chrom_pair;
+typedef unordered_map<pair<uint64_t, uint64_t>, double> Contacts;
+typedef pair<uint32_t, uint32_t> Chrom_pair;
 
-class Contact_chrom_files : public hash_map<Chrom_pair, BufferedFile *> {
+class Contact_chrom_files : public unordered_map<Chrom_pair, BufferedFile *> {
 public:
-	Contact_chrom_files() : hash_map<Chrom_pair, BufferedFile *>() {}
+	Contact_chrom_files() : unordered_map<Chrom_pair, BufferedFile *>() {}
 
 	~Contact_chrom_files() {
 		for (iterator ifile = begin(); ifile != end(); ++ifile) 
@@ -77,7 +78,7 @@ static void process_contacts_as_intervals(IntervUtils &iu, SEXP _files, Contact_
 	int64_t start1, start2, end1, end2;
 	float val;
 
-	Rprintf("Reading input file(s)...\n");
+	REprintf("Reading input file(s)...\n");
 
 	int64_t infiles_size_sum = 0;
 	for (int ifile = 0; ifile < length(_files); ++ifile) {
@@ -146,7 +147,7 @@ static void process_contacts_as_intervals(IntervUtils &iu, SEXP _files, Contact_
 			if (start1 >= end1) 
 				verror("File %s, line %ld: start1 coordinate exceeds or equals the end1 coordinate", infile.file_name().c_str(), lineno);
 
-			if (end1 > iu.get_chromkey().get_chrom_size(chromid1)) 
+			if ((uint64_t)end1 > iu.get_chromkey().get_chrom_size(chromid1)) 
 				verror("File %s, line %ld: end1 coordinate exceeds chromosome's size", infile.file_name().c_str(), lineno);
 
 			start2 = strtoll(fields[GInterval2D::START2].c_str(), &endptr, 10);
@@ -160,7 +161,7 @@ static void process_contacts_as_intervals(IntervUtils &iu, SEXP _files, Contact_
 			if (start2 >= end2) 
 				verror("File %s, line %ld: start2 coordinate exceeds or equals the end1 coordinate", infile.file_name().c_str(), lineno);
 
-			if (end2 > iu.get_chromkey().get_chrom_size(chromid2)) 
+			if ((uint64_t)end2 > iu.get_chromkey().get_chrom_size(chromid2)) 
 				verror("File %s, line %ld: end2 coordinate exceeds chromosome's size", infile.file_name().c_str(), lineno);
 
 			val = strtod(fields[GInterval2D::NUM_COLS].c_str(), &endptr);
@@ -170,7 +171,7 @@ static void process_contacts_as_intervals(IntervUtils &iu, SEXP _files, Contact_
 			int64_t coord1 = (start1 + end1) / 2;
 			int64_t coord2 = (start2 + end2) / 2;
 
-			if (chromid1 > chromid2 || chromid1 == chromid2 && coord1 > coord2) {
+			if (chromid1 > chromid2 || (chromid1 == chromid2 && coord1 > coord2)) {
 				swap(chromid1, chromid2);
 				swap(coord1, coord2);
 			}
@@ -183,7 +184,7 @@ static void process_contacts_as_intervals(IntervUtils &iu, SEXP _files, Contact_
 			if (icontact_chrom_file == contact_chrom_files.end()) {
 				char filename[FILENAME_MAX];
 
-				sprintf(filename, "%s/.%s", dirname, GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid1, chromid2).c_str());
+				snprintf(filename, sizeof(filename), "%s/.%s", dirname, GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid1, chromid2).c_str());
 				icontact_chrom_file = contact_chrom_files.insert(make_pair(chrom_pair, new BufferedFile())).first;
 				if (icontact_chrom_file->second->open(filename, "wb"))
 					verror("Writing an intermediate file %s: %s\n", filename, strerror(errno));
@@ -191,7 +192,7 @@ static void process_contacts_as_intervals(IntervUtils &iu, SEXP _files, Contact_
 
 			write_contact(*icontact_chrom_file->second, coord1, coord2, val);
 //if ((start1 + end1) / 2 == 183256 && (start2 + end2) / 2 == 214179)
-//printf("writing %g to %s\n", val, GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid1, chromid2).c_str());
+//REprintf("writing %g to %s\n", val, GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid1, chromid2).c_str());
 		}
 	}
 	progress.report_last();
@@ -224,7 +225,7 @@ static void process_contacts_as_fends(IntervUtils &iu, SEXP _contacts, SEXP _fen
 	fends_cols_total = read_header(fends_file, fends_fname, "fragment ends", FENDS_COLS, NUM_FENDS_COLS, fends_col_idx);
 
 	// read fends file
-	Rprintf("Reading fends file...\n");
+	REprintf("Reading fends file...\n");
 
 	vector<int> chromids;
 	vector<int64_t> coords;
@@ -284,11 +285,10 @@ static void process_contacts_as_fends(IntervUtils &iu, SEXP _contacts, SEXP _fen
 	progress.report_last();
 
 	// STEP 1: Read the contacts and split them into binary files each holding the contacts of specific pair of chromosomes.
-	Rprintf("Reading contacts...\n");
+	REprintf("Reading contacts...\n");
 
 	progress.init(contacts_files_size_sum, 10000000);
 
-	int64_t coord1, coord2;
 	float value;
 
 	for (int icontact_file = 0; icontact_file < length(_contacts); ++icontact_file) {
@@ -333,7 +333,7 @@ static void process_contacts_as_fends(IntervUtils &iu, SEXP _contacts, SEXP _fen
 			if (*endptr)
 				verror("Error reading contacts file %s, line %ld: invalid %s", contacts_fname, lineno, CONTACTS_COLS[COL_COUNT]);
 
-			if (chromids[fend1] > chromids[fend2] || chromids[fend1] == chromids[fend2] && coords[fend1] > coords[fend2])
+			if (chromids[fend1] > chromids[fend2] || (chromids[fend1] == chromids[fend2] && coords[fend1] > coords[fend2]))
 				swap(fend1, fend2);
 
 			Chrom_pair chrom_pair(chromids[fend1], chromids[fend2]);
@@ -342,7 +342,7 @@ static void process_contacts_as_fends(IntervUtils &iu, SEXP _contacts, SEXP _fen
 			if (icontact_chrom_file == contact_chrom_files.end()) {
 				char filename[FILENAME_MAX];
 
-				sprintf(filename, "%s/.%s", dirname, GenomeTrack::get_2d_filename(iu.get_chromkey(), chromids[fend1], chromids[fend2]).c_str());
+				snprintf(filename, sizeof(filename), "%s/.%s", dirname, GenomeTrack::get_2d_filename(iu.get_chromkey(), chromids[fend1], chromids[fend2]).c_str());
 				icontact_chrom_file = contact_chrom_files.insert(make_pair(chrom_pair, new BufferedFile())).first;
 				if (icontact_chrom_file->second->open(filename, "wb"))
 					verror("Writing an intermediate file %s: %s\n", filename, strerror(errno));
@@ -402,12 +402,12 @@ SEXP gtrack_import_contacts(SEXP _track, SEXP _contacts, SEXP _fends, SEXP _allo
 		for (Contact_chrom_files::iterator ifile = contact_chrom_files.begin(); ifile != contact_chrom_files.end(); ++ifile)
 			ifile->second->close();
 
-		Rprintf("Writing the track...\n");
+		REprintf("Writing the track...\n");
 		progress.init(contact_chrom_files.size(), 1);
 
 		for (Contact_chrom_files::iterator icontact_chrom_file = contact_chrom_files.begin(); icontact_chrom_file != contact_chrom_files.end(); ++icontact_chrom_file) {
-			size_t chromid1 = icontact_chrom_file->first.first;
-			size_t chromid2 = icontact_chrom_file->first.second;
+			uint64_t chromid1 = icontact_chrom_file->first.first;
+			uint64_t chromid2 = icontact_chrom_file->first.second;
 			BufferedFile &infile = *icontact_chrom_file->second;
 
 			// STAGE 2: Check what is the number of contacts in a chromosome pair and based on that number decide how many subtrees would be needed in StatQuadTreeCachedSerializer.
@@ -429,12 +429,12 @@ SEXP gtrack_import_contacts(SEXP _track, SEXP _contacts, SEXP _fends, SEXP _allo
 			PointsQuadTreeCachedSerializer qtree_serializer1, qtree_serializer2;
 			BufferedFiles subtrees_files(num_subtrees);
 
-			sprintf(filename, "%s/%s", dirname.c_str(), GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid1, chromid2).c_str());
+			snprintf(filename, sizeof(filename), "%s/%s", dirname.c_str(), GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid1, chromid2).c_str());
 			gtrack1.init_write(filename, chromid1, chromid2);
 			gtrack1.init_serializer(qtree_serializer1, 0, 0, iu.get_chromkey().get_chrom_size(chromid1), iu.get_chromkey().get_chrom_size(chromid2), num_subtrees, false);
 
 			if (chromid1 != chromid2) {
-				sprintf(filename, "%s/%s", dirname.c_str(), GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid2, chromid1).c_str());
+				snprintf(filename, sizeof(filename), "%s/%s", dirname.c_str(), GenomeTrack::get_2d_filename(iu.get_chromkey(), chromid2, chromid1).c_str());
 				gtrack2.init_write(filename, chromid2, chromid1);
 				gtrack2.init_serializer(qtree_serializer2, 0, 0, iu.get_chromkey().get_chrom_size(chromid2), iu.get_chromkey().get_chrom_size(chromid1), num_subtrees, false);
 			}
@@ -443,8 +443,8 @@ SEXP gtrack_import_contacts(SEXP _track, SEXP _contacts, SEXP _fends, SEXP _allo
 			if (num_subtrees > 1) {
 				const Rectangles &subarenas = qtree_serializer1.get_subarenas();
 
-				for (size_t i = 0; i < subtrees_files.size(); ++i) {
-					sprintf(filename, "%s/.%d", dirname.c_str(), i);
+				for (uint64_t i = 0; i < subtrees_files.size(); ++i) {
+					snprintf(filename, sizeof(filename), "%s/.%ld", dirname.c_str(), (long)i);
 					subtrees_files[i] = new BufferedFile();
 					if (subtrees_files[i]->open(filename, "w+")) 
 						verror("Opening an intermediate file %s: %s\n", filename, strerror(errno));
@@ -482,14 +482,14 @@ SEXP gtrack_import_contacts(SEXP _track, SEXP _contacts, SEXP _fends, SEXP _allo
 					check_interrupt();
 				}
 
-				for (size_t i = 0; i < subtrees_files.size(); ++i)
+				for (uint64_t i = 0; i < subtrees_files.size(); ++i)
 					subtrees_files[i]->seek(0, SEEK_SET);
 
 				infile.close();
 				unlink(infile.file_name().c_str());
 			}
 
-			for (size_t i = 0; i < num_subtrees; ++i) {
+			for (uint64_t i = 0; i < (uint64_t)num_subtrees; ++i) {
 				BufferedFile &subtree_file = num_subtrees == 1 ? infile : *subtrees_files[i];
 				Contacts contacts;
 
@@ -504,7 +504,7 @@ SEXP gtrack_import_contacts(SEXP _track, SEXP _contacts, SEXP _fends, SEXP _allo
 					if (subtree_file.error()) 
 						verror("Reading file %s: %s\n", subtree_file.file_name().c_str(), strerror(errno));
 
-					pair<size_t, size_t> key(coord1, coord2);
+					pair<uint64_t, uint64_t> key(coord1, coord2);
 					Contacts::iterator icontact = contacts.find(key);
 
 					if (icontact == contacts.end())
@@ -549,7 +549,9 @@ SEXP gtrack_import_contacts(SEXP _track, SEXP _contacts, SEXP _fends, SEXP _allo
 		progress.report_last();
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 
 	return R_NilValue;
 }
