@@ -5,11 +5,12 @@
  *      Author: hoichman
  */
 
+#include <cstdint>
 #include <errno.h>
-#include <sys/timeb.h>
+#include <sys/time.h>
 
 #include <set>
-#include <ext/hash_set>
+#include <unordered_set>
 
 #include "port.h"
 
@@ -33,7 +34,6 @@
 #include "TrackExpressionScanner.h"
 #include "TrackExpressionTrackRectsIterator.h"
 #include "TrackExpressionVars.h"
-
 #include "rdbutils.h"
 
 const int TrackExprScanner::INIT_REPORT_STEP = 10000;
@@ -44,15 +44,13 @@ using namespace rdb;
 
 static uint64_t get_cur_clock()
 {
-	struct timeb tp;
-	ftime(&tp);
-	return tp.time * 1000 + tp.millitm;
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	return tv.tv_sec * 1000 + tv.tv_usec / 1000;
 }
 
 TrackExprScanner::TrackExprScanner(rdb::IntervUtils &iu) :
-	m_iu(iu),
-	m_num_track_vars(0),
-	m_num_interv_vars(0),
+	m_iu(iu),	
 	m_isend(true),
 	m_expr_itr(NULL),
 	m_expr_vars(iu)
@@ -112,7 +110,13 @@ void TrackExprScanner::define_r_vars(unsigned eval_buf_limit)
 		for (unsigned i = 0; i < m_eval_buf_limit; i++)
 			m_2d.expr_itr_intervals_chroms1[i] = m_2d.expr_itr_intervals_chroms2[i] = 1;
 	}
-	defineVar(install("GITERATOR.INTERVALS"), m_rexpr_itr_intervals, findVar(install(".GlobalEnv"), m_iu.get_env()));
+	defineVar(install("GITERATOR.INTERVALS"), m_rexpr_itr_intervals, findVar(install(".misha"), m_iu.get_env()));
+
+    for (unsigned iexpr = 0; iexpr < m_track_exprs.size(); ++iexpr) {
+        const TrackExpressionVars::Track_var *var = m_expr_vars.var(m_track_exprs[iexpr].c_str());
+        if (var)  // track expression is a virtual track
+            m_eval_doubles[iexpr] = REAL(var->rvar);
+    }
 }
 
 void TrackExprScanner::check(string &track_expr, GIntervalsFetcher1D *scope1d, GIntervalsFetcher2D *scope2d, SEXP iterator_policy, SEXP band)
@@ -129,16 +133,33 @@ void TrackExprScanner::check(SEXP track_exprs, GIntervalsFetcher1D *scope1d, GIn
 
 void TrackExprScanner::check(const vector<string> &track_exprs, GIntervalsFetcher1D *scope1d, GIntervalsFetcher2D *scope2d, SEXP iterator_policy, SEXP band)
 {
-	m_track_exprs = track_exprs;
 	m_band = m_iu.convert_band(band);
 
-	runprotect(m_eval_exprs);
-	m_eval_exprs.resize(m_track_exprs.size(), R_NilValue);
+    runprotect(m_eval_bufs);
+    runprotect(m_eval_exprs);
 
-	runprotect(m_eval_bufs);
-	m_eval_bufs.resize(m_track_exprs.size(), R_NilValue);
-	m_eval_doubles.resize(m_track_exprs.size(), NULL);
-	m_eval_ints.resize(m_track_exprs.size(), NULL);
+    m_track_exprs.reserve(track_exprs.size());
+    for (vector<string>::const_iterator iexpr = track_exprs.begin(); iexpr != track_exprs.end(); ++iexpr) {
+        // trim spaces from the track expression
+        string::const_iterator istr_start, istr_end;
+
+        for (istr_start = iexpr->begin(); istr_start < iexpr->end(); ++istr_start) {
+            if (!isspace(*istr_start))
+                break;
+        }
+
+        for (istr_end = iexpr->end() - 1; istr_end >= iexpr->begin(); --istr_end) {
+            if (!isspace(*istr_end))
+                break;
+        }
+
+        m_track_exprs.push_back(iexpr->substr(istr_start - iexpr->begin(), istr_end - istr_start + 1));
+    }
+
+    m_eval_exprs.resize(m_track_exprs.size(), R_NilValue);
+    m_eval_bufs.resize(m_track_exprs.size(), R_NilValue);
+    m_eval_doubles.resize(m_track_exprs.size(), NULL);
+    m_eval_ints.resize(m_track_exprs.size(), NULL);
 
 	m_expr_vars.parse_exprs(m_track_exprs);
 
@@ -148,18 +169,21 @@ void TrackExprScanner::check(const vector<string> &track_exprs, GIntervalsFetche
 	m_expr_vars.init(*m_expr_itr);
 
 	for (unsigned iexpr = 0; iexpr < m_track_exprs.size(); ++iexpr) {
-		SEXP expr;
-		rprotect(expr = allocVector(STRSXP, 1));
-		SET_STRING_ELT(expr, 0, mkChar(m_track_exprs[iexpr].c_str()));
+        if (!m_expr_vars.var(m_track_exprs[iexpr].c_str())) {   // track expression is not a virtual track
+    		SEXP expr = R_NilValue;
+			SEXPCleaner expr_cleaner(expr);
 
-		// parse R expression
-		ParseStatus status;
-		SEXP parsed_expr;
-		rprotect(parsed_expr = R_ParseVector(expr, -1, &status, R_NilValue));
-		if (status != PARSE_OK)
-			verror("R parsing of expression \"%s\" failed", m_track_exprs[iexpr].c_str());
-		m_eval_exprs[iexpr] = VECTOR_ELT(parsed_expr, 0);
-		runprotect(expr);
+    		rprotect(expr = RSaneAllocVector(STRSXP, 1));
+    		SET_STRING_ELT(expr, 0, mkChar(m_track_exprs[iexpr].c_str()));
+
+    		// parse R expression
+    		ParseStatus status;
+    		SEXP parsed_expr;
+    		rprotect(parsed_expr = R_ParseVector(expr, -1, &status, R_NilValue));
+    		if (status != PARSE_OK)
+    			verror("R parsing of expression \"%s\" failed", m_track_exprs[iexpr].c_str());
+    		m_eval_exprs[iexpr] = VECTOR_ELT(parsed_expr, 0);
+        }
 	}
 }
 
@@ -187,14 +211,16 @@ bool TrackExprScanner::begin(const vector<string> &track_exprs, GIntervalsFetche
 		define_r_vars((unsigned)REAL(gbufsize)[0]);
 
 	for (unsigned iexpr = 0; iexpr < m_track_exprs.size(); ++iexpr) {
-		SEXP res = eval_in_R(m_eval_exprs[iexpr], m_iu.get_env());
+        if (m_eval_exprs[iexpr] != R_NilValue) {
+            SEXP res = eval_in_R(m_eval_exprs[iexpr], m_iu.get_env());
 
-		if (length(res) != (int)m_eval_buf_limit) {
-			runprotect(res);
-			define_r_vars(1);
-			break;
-		}
-		runprotect(res);
+            if (length(res) != (int)m_eval_buf_limit) {
+                runprotect(res);
+                define_r_vars(1);
+                break;
+            }
+            runprotect(res);
+        }
 	}
 
 	m_num_evals = 0;
@@ -213,6 +239,8 @@ bool TrackExprScanner::begin(const vector<string> &track_exprs, GIntervalsFetche
 
 bool TrackExprScanner::next()
 {
+	monitor_memusage();
+
 	if (isend())
 		return false;
 
@@ -222,9 +250,9 @@ bool TrackExprScanner::next()
 	// did we start reporting progress?
 	if (m_last_progress_reported >= 0) {
 		if (m_last_progress_reported != 100)
-			Rprintf("100%%\n");
+			REprintf("100%%\n");
 		else
-			Rprintf("\n");
+			REprintf("\n");
 	}
 	update_progress(100);
 
@@ -242,28 +270,28 @@ bool TrackExprScanner::eval_next()
 		m_eval_buf_idx = 0;
 		if (m_expr_itr->is_1d()) {
 			for (m_eval_buf_size = 0; m_eval_buf_size < m_eval_buf_limit; ++m_eval_buf_size) {
-				if (m_expr_itr->isend()) {
-					for (unsigned i = m_eval_buf_size; i < m_eval_buf_limit; ++i) {
-						m_1d.expr_itr_intervals_chroms[i] = 1;
-						m_1d.expr_itr_intervals_starts[i] = -1.;
-						m_1d.expr_itr_intervals_ends[i] = -1.;
-					}
-					break;
-				}
+                if (m_expr_itr->isend()) {
+                    for (unsigned i = m_eval_buf_size; i < m_eval_buf_limit; ++i) {
+                        m_1d.expr_itr_intervals_chroms[i] = 1;
+                        m_1d.expr_itr_intervals_starts[i] = -1.;
+                        m_1d.expr_itr_intervals_ends[i] = -1.;
+                    }
+                    break;
+                }
 
-				const GInterval &interval = ((TrackExpression1DIterator *)m_expr_itr)->last_interval();
+                const GInterval &interval = ((TrackExpression1DIterator *)m_expr_itr)->last_interval();
 
-				m_1d.expr_itr_intervals[m_eval_buf_size] = interval;
-				m_1d.expr_itr_scope_intervals[m_eval_buf_size] = ((TrackExpression1DIterator *)m_expr_itr)->last_scope_interval();
-				m_1d.expr_itr_intervals_chroms[m_eval_buf_size] = interval.chromid + 1;
-				m_1d.expr_itr_intervals_starts[m_eval_buf_size] = (double)interval.start;
-				m_1d.expr_itr_intervals_ends[m_eval_buf_size] = (double)interval.end;
-				m_1d.cur_chromid = interval.chromid;
+                m_1d.expr_itr_intervals[m_eval_buf_size] = interval;
+                m_1d.expr_itr_scope_intervals[m_eval_buf_size] = ((TrackExpression1DIterator *)m_expr_itr)->last_scope_interval();
+                m_1d.expr_itr_intervals_chroms[m_eval_buf_size] = interval.chromid + 1;
+                m_1d.expr_itr_intervals_starts[m_eval_buf_size] = (double)interval.start;
+                m_1d.expr_itr_intervals_ends[m_eval_buf_size] = (double)interval.end;
+                m_1d.cur_chromid = interval.chromid;
 
-				m_expr_itr_scope_idx[m_eval_buf_size] = ((TrackExpression1DIterator *)m_expr_itr)->get_cur_scope_idx();
-				m_expr_itr_scope_chrom_idx[m_eval_buf_size] = ((TrackExpression1DIterator *)m_expr_itr)->get_cur_scope_chrom_idx();
-				m_expr_vars.set_vars(interval, m_eval_buf_size);
-				m_expr_itr->next();
+                m_expr_itr_scope_idx[m_eval_buf_size] = ((TrackExpression1DIterator *)m_expr_itr)->get_cur_scope_idx();
+                m_expr_itr_scope_chrom_idx[m_eval_buf_size] = ((TrackExpression1DIterator *)m_expr_itr)->get_cur_scope_chrom_idx();
+                m_expr_vars.set_vars(interval, m_eval_buf_size);
+                m_expr_itr->next();
 			}
 		}
 		else {
@@ -300,22 +328,24 @@ bool TrackExprScanner::eval_next()
 			}
 		}
 
-		check_interrupt();
+        check_interrupt();
 
-		runprotect(m_eval_bufs);
-		for (unsigned iexpr = 0; iexpr < m_eval_exprs.size(); ++iexpr) {
-			m_eval_bufs[iexpr] = eval_in_R(m_eval_exprs[iexpr], m_iu.get_env());
-			if (length(m_eval_bufs[iexpr]) != (int)m_eval_buf_limit)
-				verror("Evaluation of expression \"%s\" produces a vector of size %d while expecting size %d",
-						m_track_exprs[iexpr].c_str(), length(m_eval_bufs[iexpr]), m_eval_buf_limit);
-			if (isReal(m_eval_bufs[iexpr]))
-				m_eval_doubles[iexpr] = REAL(m_eval_bufs[iexpr]);
-			else if (isLogical(m_eval_bufs[iexpr])) 
-				m_eval_ints[iexpr] = LOGICAL(m_eval_bufs[iexpr]);
-			else
-				verror("Evaluation of expression \"%s\" produces a vector of unsupported type %s",
-						m_track_exprs[iexpr].c_str(), type2char(TYPEOF(m_eval_bufs[iexpr])));
-		}
+        for (unsigned iexpr = 0; iexpr < m_eval_exprs.size(); ++iexpr) {
+            if (m_eval_exprs[iexpr] != R_NilValue) {
+                runprotect(m_eval_bufs[iexpr]);
+                m_eval_bufs[iexpr] = eval_in_R(m_eval_exprs[iexpr], m_iu.get_env());
+                if (length(m_eval_bufs[iexpr]) != (int)m_eval_buf_limit)
+                    verror("Evaluation of expression \"%s\" produces a vector of size %d while expecting size %d",
+                            m_track_exprs[iexpr].c_str(), length(m_eval_bufs[iexpr]), m_eval_buf_limit);
+                if (isReal(m_eval_bufs[iexpr]))
+                    m_eval_doubles[iexpr] = REAL(m_eval_bufs[iexpr]);
+                else if (isLogical(m_eval_bufs[iexpr]))
+                    m_eval_ints[iexpr] = LOGICAL(m_eval_bufs[iexpr]);
+                else
+                    verror("Evaluation of expression \"%s\" produces a vector of unsupported type %s",
+                            m_track_exprs[iexpr].c_str(), type2char(TYPEOF(m_eval_bufs[iexpr])));
+            }
+        }
 
 		report_progress();
 	}
@@ -328,81 +358,43 @@ bool TrackExprScanner::eval_next()
 	return !m_isend;
 }
 
-double TrackExprScanner::last_real(int idx) const
-{
-	SEXP expr = m_eval_bufs[idx];
-	double val = -1;
-
-	if (expr != R_NilValue) {
-		if (!isReal(expr)) {
-			if (RdbInitializer::is_kid()) 
-				verror("Expression \"%s\" does not produce a numeric result.", m_track_exprs[idx].c_str());
-			else {
-				defineVar(install("GERROR_EXPR"), expr, findVar(install(".GlobalEnv"), m_iu.get_env()));
-				verror("Expression \"%s\" does not produce a numeric result.\n"
-						"The result of the last expression evaluation was saved in GERROR_EXPR variable.", m_track_exprs[idx].c_str());
-			}
-		}
-		val = m_eval_doubles[idx][m_eval_buf_idx];
-	}
-	return val;
-}
-
-int TrackExprScanner::last_logical(int idx) const
-{
-	SEXP expr = m_eval_bufs[idx];
-	int val = -1;
-
-	if (expr != R_NilValue) {
-		if (!isLogical(expr)) {
-			if (RdbInitializer::is_kid()) 
-				verror("Expression \"%s\" does not produce a logical result\n", m_track_exprs[idx].c_str());
-			else {
-				defineVar(install("GERROR_EXPR"), expr, findVar(install(".GlobalEnv"), m_iu.get_env()));
-				verror("Expression \"%s\" does not produce a logical result\n"
-						"The result of the last expression evaluation was saved in GERROR_EXPR variable.", m_track_exprs[idx].c_str());
-			}
-		}
-		val = m_eval_ints[idx][m_eval_buf_idx];
-	}
-	return val;
-}
-
 void TrackExprScanner::report_progress()
 {
 	m_num_evals += m_eval_buf_size;
-	if (m_num_evals > (size_t)m_report_step && m_do_report_progress) {
-		uint64_t curclock = get_cur_clock();
-		double delta = curclock - m_last_report_clock;
+	if (m_num_evals > (uint64_t)m_report_step && m_do_report_progress) {
+        uint64_t curclock = get_cur_clock();
+        double delta = curclock - m_last_report_clock;
 
-		if (delta)
-			m_report_step = (int)(m_report_step * (REPORT_INTERVAL / delta) + .5);
-		else
-			m_report_step *= 10;
+        if (delta)
+            m_report_step = (int)(m_report_step * (REPORT_INTERVAL / delta) + .5);
+        else
+            m_report_step *= 10;
 
-		if (delta > MIN_REPORT_INTERVAL) {
-			if (m_last_progress_reported < 0 && m_eval_buf_limit == 1)
-				Rprintf("Warning: track expression(s) cannot be evaluated as a vector. Run-times might be slow.\n");
+        if (delta > MIN_REPORT_INTERVAL) {
+            if (m_last_progress_reported < 0 && m_eval_buf_limit == 1)
+                REprintf("Warning: track expression(s) cannot be evaluated as a vector. Run-times might be slow.\n");
 
-			int progress;
+            int progress;
 
-			if (m_expr_itr->is_1d())
-				progress = !((TrackExpression1DIterator *)m_expr_itr)->get_scope()->size() ? 0 :
-					(int)(100. * (((TrackExpression1DIterator *)m_expr_itr)->get_cur_scope_idx()) / ((TrackExpression1DIterator *)m_expr_itr)->get_scope()->size());
-			else
-				progress = !((TrackExpression2DIterator *)m_expr_itr)->get_scope()->size() ? 0 :
-					(int)(100. * (((TrackExpression2DIterator *)m_expr_itr)->get_cur_scope_idx()) / ((TrackExpression2DIterator *)m_expr_itr)->get_scope()->size());
+            if (m_expr_itr->is_1d())
+                progress = !((TrackExpression1DIterator *)m_expr_itr)->get_scope()->size() ? 0 :
+                    (int)(100. * (((TrackExpression1DIterator *)m_expr_itr)->get_cur_scope_idx()) / ((TrackExpression1DIterator *)m_expr_itr)->get_scope()->size());
+            else
+                progress = !((TrackExpression2DIterator *)m_expr_itr)->get_scope()->size() ? 0 :
+                    (int)(100. * (((TrackExpression2DIterator *)m_expr_itr)->get_cur_scope_idx()) / ((TrackExpression2DIterator *)m_expr_itr)->get_scope()->size());
 
-			progress = max(progress, m_last_progress_reported);  // in 2D scope idx can sometimes go backward
-			if (progress != m_last_progress_reported) {
-				Rprintf("%d%%...", progress);
-				update_progress(progress);
-			} else
-				Rprintf(".");
-			m_last_progress_reported = progress;
-			m_num_evals = 0;
-			m_last_report_clock = curclock;
-		}
+            progress = max(progress, m_last_progress_reported);  // in 2D scope idx can sometimes go backward
+            if (progress != 100) {
+                if (progress != m_last_progress_reported) {
+                    REprintf("%d%%...", progress);
+                    update_progress(progress);
+                } else
+                    REprintf(".");
+                m_last_progress_reported = progress;
+            }
+            m_num_evals = 0;
+            m_last_report_clock = curclock;
+        }
 	}
 }
 
@@ -437,11 +429,14 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP rtrack_
 	// initiate the expression iterator
 	delete m_expr_itr;
 	m_expr_itr = create_expr_iterator(iterator_policy, m_expr_vars, m_track_exprs, scope1d, scope2d, m_1d.intervals, m_2d.intervals, m_band, call_begin);
+    return m_expr_itr;
 }
 
 TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterator, const TrackExpressionVars &vars, const vector<string> &track_exprs,
 		GIntervalsFetcher1D *scope1d, GIntervalsFetcher2D *scope2d, GIntervals &intervals1d, GIntervals2D &intervals2d, const DiagonalBand &band, bool call_begin)
 {
+	monitor_memusage();
+
 	TrackExpressionIteratorBase *expr_itr = NULL;
 	SEXP class_name = getAttrib(giterator, R_ClassSymbol);
 
@@ -470,7 +465,7 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 		SEXP rexpansion1 = VECTOR_ELT(giterator, 2);
 		SEXP rexpansion2 = VECTOR_ELT(giterator, 3);
 
-		if (!isReal(rexpansion1) && !isInteger(rexpansion1) || !isNull(rexpansion2) && !isReal(rexpansion2) && !isInteger(rexpansion2))
+		if ((!isReal(rexpansion1) && !isInteger(rexpansion1)) || (!isNull(rexpansion2) && !isReal(rexpansion2) && !isInteger(rexpansion2)))
 			verror("Invalid format of cartesian grid iterator");
 
 		vector<int64_t> expansion1(length(rexpansion1));
@@ -488,7 +483,7 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 
 		SEXP rband_idx = VECTOR_ELT(giterator, 4);
 
-		if (!isReal(rband_idx) && !isInteger(rband_idx) || length(rband_idx) != 3)
+		if ((!isReal(rband_idx) && !isInteger(rband_idx)) || length(rband_idx) != 3)
 			verror("Invalid format of cartesian grid iterator");
 
 		int64_t min_band_idx = isReal(rband_idx) ? (int64_t)REAL(rband_idx)[0] : INTEGER(rband_idx)[0];
@@ -502,7 +497,7 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 																	  m_iu.get_max_data_size());
 	} else if (isString(giterator) && length(giterator) == 1 && GIntervalsBigSet::isbig(CHAR(STRING_ELT(giterator, 0)), m_iu)) {
 		const char *intervset = CHAR(STRING_ELT(giterator, 0));
-		SEXP meta = GIntervalsBigSet::get_meta(intervset, m_iu);
+		SEXP meta = GIntervalsMeta::load_meta(interv2path(m_iu.get_env(), intervset).c_str());
 
 		if (GIntervalsBigSet1D::is1d(meta)) {
 			expr_itr = new TrackExpressionBigSet1DIterator(m_iu);
@@ -514,7 +509,9 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 				((TrackExpressionBigSet2DIterator *)expr_itr)->begin(intervset, meta, *scope2d, band, m_iu.get_max_data_size());
 		}
 		runprotect(meta);
-	} else if (isVector(giterator) || isString(giterator) && length(giterator) == 1) {   // giterator == intervals
+	} else if ((isVector(giterator) && !isString(giterator)) ||
+			   (isString(giterator) && length(giterator) == 1 && !m_iu.track_exists(CHAR(STRING_ELT(giterator, 0)))))
+	{   // giterator == intervals
 		unsigned intervs_type_mask;
 		if (m_iu.convert_rintervs(giterator, &intervals1d, &intervals2d, true, NULL, "", &intervs_type_mask)) {
 			if (intervs_type_mask == (IntervUtils::INTERVS1D | IntervUtils::INTERVS2D))
@@ -559,8 +556,9 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 		if (find(track_names.begin(), track_names.end(), iter_val) == track_names.end()) {
 			if (isString(giterator)) {
 				SEXP all_track_names;
+				SEXPCleaner all_track_names_cleaner(all_track_names);
 
-				rprotect(all_track_names = findVar(install("GTRACKS"), m_iu.get_env()));
+				rprotect(all_track_names = findVar(install("GTRACKS"), findVar(install(".misha"), m_iu.get_env())));
 				if (isString(all_track_names)) {
 					int i;
 					for (i = 0; i < length(all_track_names); ++i) {
@@ -570,7 +568,7 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 					if (i >= length(all_track_names)) 
 						verror("Invalid iterator: %s is neither a name of a track nor a name of an intervals set", iter_val.c_str());
 				}
-			}
+                        }
 			track_names.push_back(iter_val);
 			track_types.push_back(GenomeTrack::get_type(track2path(m_iu.get_env(), iter_val).c_str(), m_iu.get_chromkey()));
 		}
@@ -590,8 +588,8 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 			if (GenomeTrack::is_1d(track_type)) {
 				set<int> chromids;
 
-				for (vector<string>::const_iterator ifilename = filenames.begin(); ifilename != filenames.end(); ++ifilename) {
-					int chromid;
+				for (vector<string>::const_iterator ifilename = filenames.begin(); ifilename != filenames.end(); ++ifilename) {										
+					int chromid = -1;
 					GenomeTrackFixedBin gtrack_fbin;
 
 					try {
@@ -602,12 +600,13 @@ TrackExpressionIteratorBase *TrackExprScanner::create_expr_iterator(SEXP giterat
 
 					chromids.insert(chromid);
 					if (track_type == GenomeTrack::FIXED_BIN) {
-						gtrack_fbin.init_read((trackpath + "/" + *ifilename).c_str(), chromid);
-						int64_t expected_num_bins = (int64_t)ceil(all_genome_intervs[chromid].end / (double)gtrack_fbin.get_bin_size());
+                        gtrack_fbin.init_read((trackpath + "/" + *ifilename).c_str(), chromid);
+                        int64_t expected_num_bins = (int64_t)ceil(all_genome_intervs[chromid].end / (double)gtrack_fbin.get_bin_size());
 
-						if (gtrack_fbin.get_num_samples() != expected_num_bins)
-							verror("Number of bins in track %s, chrom %s do not match the chromosome size (expecting: %ld, reading: %ld)",
-									itrack_name->c_str(), m_iu.get_chromkey().id2chrom(chromid).c_str(), expected_num_bins, gtrack_fbin.get_num_samples());
+                        if (gtrack_fbin.get_num_samples() != expected_num_bins){
+                            verror("Number of bins in track %s, chrom %s do not match the chromosome size (expecting: %ld, reading: %ld)",
+                                    itrack_name->c_str(), m_iu.get_chromkey().id2chrom(chromid).c_str(), expected_num_bins, gtrack_fbin.get_num_samples());
+						}
 					}
 
 					if (ifilename == filenames.begin()) {
