@@ -5,10 +5,12 @@
  *      Author: hoichman
  */
 
+#include <cstdint>
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <memory>
 #include <new>
 #include <setjmp.h>
 #include <stdarg.h>
@@ -24,6 +26,11 @@
 	#undef _POSIX_C_SOURCE
 #endif
 
+#if defined(__APPLE__)
+    #include <libproc.h>
+    #include <sys/proc_info.h>
+#endif
+
 #include "rdbinterval.h"
 #include "rdbutils.h"
 #include "System.h"
@@ -38,61 +45,60 @@
 
 using namespace std;
 using namespace rdb;
-using namespace __gnu_cxx;
 
-// A hack into R to change the default error report mechanism.
+//// A hack into R to change the default error report mechanism.
+////
+//// Surely this an ugly hack! R_GlobalContext is an internal object managed by R. Its type is not even exposed. RCNTXT structure is defined
+//// in an internal header file (src/include/Defn.h) that we cannot access. We copied the definition of it from there and changed it a bit
+//// to compile.
 //
-// Surely this an ugly hack! R_GlobalContext is an internal object managed by R. Its type is not even exposed. RCNTXT structure is defined
-// in an internal header file (src/include/Defn.h) that we cannot access. We copied the definition of it from there and changed it a bit
-// to compile.
-
-typedef struct RCNTXT {
-    struct RCNTXT *nextcontext;	/* The next context up the chain */
-    int callflag;		/* The context "type" */
-    sigjmp_buf cjmpbuf;		/* C stack and register information */
-    int cstacktop;		/* Top of the pointer protection stack */
-    int evaldepth;	        /* evaluation depth at inception */
-    SEXP promargs;		/* Promises supplied to closure */
-    SEXP callfun;		/* The closure called */
-    SEXP sysparent;		/* environment the closure was called from */
-    SEXP call;			/* The call that effected this context*/
-    SEXP cloenv;		/* The environment */
-    SEXP conexit;		/* Interpreted "on.exit" code */
-    void (*cend)(void *);	/* C "on.exit" thunk */
-    void *cenddata;		/* data for C "on.exit" thunk */
-    void *vmax;		        /* top of R_alloc stack */
-    int intsusp;                /* interrupts are suspended */
-    SEXP handlerstack;          /* condition handler stack */
-    SEXP restartstack;          /* stack of available restarts */
-    struct RPRSTACK *prstack;   /* stack of pending promises */
-#ifdef BYTECODE
-    SEXP *nodestack;
-# ifdef BC_INT_STACK
-    IStackval *intstack;
-# endif
-#endif
-    SEXP srcref;	        /* The source line in effect */
-} RCNTXT, *context;
+//typedef struct RCNTXT {
+//    struct RCNTXT *nextcontext; /* The next context up the chain */
+//    int callflag;       /* The context "type" */
+//    sigjmp_buf cjmpbuf;     /* C stack and register information */
+//    int cstacktop;      /* Top of the pointer protection stack */
+//    int evaldepth;          /* evaluation depth at inception */
+//    SEXP promargs;      /* Promises supplied to closure */
+//    SEXP callfun;       /* The closure called */
+//    SEXP sysparent;     /* environment the closure was called from */
+//    SEXP call;          /* The call that effected this context*/
+//    SEXP cloenv;        /* The environment */
+//    SEXP conexit;       /* Interpreted "on.exit" code */
+//    void (*cend)(void *);   /* C "on.exit" thunk */
+//    void *cenddata;     /* data for C "on.exit" thunk */
+//    void *vmax;             /* top of R_alloc stack */
+//    int intsusp;                /* interrupts are suspended */
+//    SEXP handlerstack;          /* condition handler stack */
+//    SEXP restartstack;          /* stack of available restarts */
+//    struct RPRSTACK *prstack;   /* stack of pending promises */
+//#ifdef BYTECODE
+//    SEXP *nodestack;
+//# ifdef BC_INT_STACK
+//    IStackval *intstack;
+//# endif
+//#endif
+//    SEXP srcref;            /* The source line in effect */
+//} RCNTXT, *context;
 
 static const char *CHROM_FILE_PREFIX = "chr";
 static const unsigned CHROM_FILE_PREFIX_LEN = strlen(CHROM_FILE_PREFIX);
 const string rdb::TRACK_FILE_EXT = ".track";
 const string rdb::INTERV_FILE_EXT = ".interv";
 
-size_t               RdbInitializer::s_shm_size;
-size_t               RdbInitializer::s_res_const_size;
-size_t               RdbInitializer::s_res_var_size;
-size_t               RdbInitializer::s_max_res_size;
-size_t               RdbInitializer::s_max_mem_usage;
-size_t               RdbInitializer::s_init_mem_usage;
+const int64_t        RdbInitializer::LAUNCH_DELAY = 50; // in msec
+const int64_t        RdbInitializer::MEM_SYNC_DELAY = 100;
+const int64_t        RdbInitializer::REPORT_INTERVAL_DELAY = 3000;
+uint64_t               RdbInitializer::s_shm_size;
+uint64_t               RdbInitializer::s_res_const_size;
+uint64_t               RdbInitializer::s_res_var_size;
+uint64_t               RdbInitializer::s_max_res_size;
+uint64_t               RdbInitializer::s_max_mem_usage;
 bool                 RdbInitializer::s_is_kid = false;
 pid_t                RdbInitializer::s_parent_pid = 0;
-sem_t                RdbInitializer::s_signal_sem;
 sem_t               *RdbInitializer::s_shm_sem = SEM_FAILED;
 sem_t               *RdbInitializer::s_alloc_suspend_sem = SEM_FAILED;
 int                  RdbInitializer::s_kid_index;
 vector<RdbInitializer::LiveStat>     RdbInitializer::s_running_pids;
-vector<RdbInitializer::DeathStat>    RdbInitializer::s_dead_pids;
 RdbInitializer::Shm *RdbInitializer::s_shm = (RdbInitializer::Shm *)MAP_FAILED;
 struct sigaction     RdbInitializer::s_old_sigint_act;
 struct sigaction     RdbInitializer::s_old_sigchld_act;
@@ -114,24 +120,14 @@ RdbInitializer::RdbInitializer()
 		s_shm_size = 0;
 		s_is_kid = false;
 		s_parent_pid = getpid();
-		sem_init(&s_signal_sem, 0, 0);
 		s_shm_sem = SEM_FAILED;
 		s_alloc_suspend_sem = SEM_FAILED;
 		s_shm = (Shm *)MAP_FAILED;
 		s_kid_index = 0;
 		s_running_pids.clear();
-		s_dead_pids.clear();
-
-		// These containers are altered in a signal handler of SIGCHLD.
-		// Since malloc should be avoided in a signal handler let's reserve the memory in advance.
-		s_running_pids.reserve(MAX_KIDS);
-		s_dead_pids.reserve(MAX_KIDS);
 
 		m_old_error_handler = TGLException::set_error_handler(TGLException::throw_error_handler);
 
-		// install out of memory handler
-		m_old_new_handler = set_new_handler(out_of_memory);
-
 		struct sigaction new_act;
 
 		// install a new SIGINT handler
@@ -150,15 +146,15 @@ RdbInitializer::RdbInitializer()
 		get_open_fds(m_old_open_fds);
 	}
 
-	// Default error message that error() function generates includes the caller ("Error in long-blalalalalalala: ...")
-	// To prevent the caller to be printed we simply screw it up.
-	// (This ugly hack was made after learning verrorcall_dflt() function in R source code (error.c).)
-	RCNTXT *c = (RCNTXT *)R_GlobalContext;
-	if (c) {
-		c->call = R_NilValue;
-		if (c->nextcontext)
-			c->nextcontext->call = R_NilValue;
-	}
+//  // Default error message that error() function generates includes the caller ("Error in long-blalalalalalala: ...")
+//  // To prevent the caller to be printed we simply screw it up.
+//  // (This ugly hack was made after learning verrorcall_dflt() function in R source code (error.c).)
+//  RCNTXT *c = (RCNTXT *)R_GlobalContext;
+//  if (c) {
+//  	c->call = R_NilValue;
+//  	if (c->nextcontext)
+//  		c->nextcontext->call = R_NilValue;
+//  }
 
 	s_ref_count++;
 
@@ -177,22 +173,22 @@ RdbInitializer::~RdbInitializer()
 		if (!s_is_kid) {
 			if (s_shm_sem != SEM_FAILED) {
 				SemLocker sl(s_shm_sem);
-				SigchldBlocker sb;
+				SigBlocker sb;
 
 				// kill all the remaining child processes
-				for (vector<LiveStat>::const_iterator ipid = s_running_pids.begin(); ipid != s_running_pids.end(); ++ipid) 
+				for (vector<LiveStat>::const_iterator ipid = s_running_pids.begin(); ipid != s_running_pids.end(); ++ipid)
 					kill(ipid->pid, SIGTERM);
 			}
 
 			// after SIGTERM is sent to all the kids let's wait till sigchld_hander() burries them all
-			while (1) {
-				{
-					SigchldBlocker sb;
-					if (s_running_pids.empty()) 
-						break;
-				}
-				sem_wait(&s_signal_sem);
-			}
+            while (1) {
+                SigBlocker sb;
+                check_kids_state(true);
+                if (s_running_pids.empty())
+                    break;
+
+                sigsuspend(&sb.oldsigset);
+            }
 
 			if (s_shm_sem != SEM_FAILED)
 				sem_close(s_shm_sem); // semaphore should be already unlinked, only need to close it
@@ -206,23 +202,17 @@ RdbInitializer::~RdbInitializer()
 
 		TGLException::set_error_handler(m_old_error_handler);
 
-		// install old out of memory handler
-		if (m_old_new_handler)
-			set_new_handler(m_old_new_handler);
-
 		// install old signal handlers
 		sigaction(SIGINT, &s_old_sigint_act, NULL);
 		sigaction(SIGCHLD, &s_old_sigchld_act, NULL);
 
-		sem_destroy(&s_signal_sem);
-
-		// close all file descriptors opened during the session
-		set<int> open_fds;
-		get_open_fds(open_fds);
-		for (set<int>::const_iterator ifd = open_fds.begin(); ifd != open_fds.end(); ++ifd) {
-			if (m_old_open_fds.find(*ifd) == m_old_open_fds.end())
-				close(*ifd);
-		}
+        // close all file descriptors opened during the session
+        set<int> open_fds;
+        get_open_fds(open_fds);
+        for (set<int>::const_iterator ifd = open_fds.begin(); ifd != open_fds.end(); ++ifd) {
+            if (m_old_open_fds.find(*ifd) == m_old_open_fds.end())
+                close(*ifd);
+        }
 
 		umask(m_old_umask);
 
@@ -237,19 +227,22 @@ RdbInitializer::~RdbInitializer()
 string RdbInitializer::get_shm_sem_name()
 {
 	char buf[100];
-	sprintf(buf, "misha-shm-%d", (int)getpid());
+	snprintf(buf, sizeof(buf), "misha-shm-%d", (int)getpid());
 	return buf;
 }
 
 string RdbInitializer::get_alloc_suspend_sem_name()
 {
 	char buf[100];
-	sprintf(buf, "misha-alloc-suspend-%d", (int)getpid());
+	snprintf(buf, sizeof(buf), "misha-alloc-suspend-%d", (int)getpid());
 	return buf;
 }
 
-void RdbInitializer::prepare4multitasking(size_t res_const_size, size_t res_var_size, size_t max_res_size, size_t max_mem_usage, unsigned num_planned_kids)
+void RdbInitializer::prepare4multitasking(uint64_t res_const_size, uint64_t res_var_size, uint64_t max_res_size, uint64_t max_mem_usage, unsigned num_planned_kids)
 {
+	if (num_planned_kids > MAX_KIDS) 
+		verror("Too many child processes");
+
 	if (s_shm_sem == SEM_FAILED) {
 		sem_unlink(get_shm_sem_name().c_str()); // remove a semaphore if it was somehow not cleaned from the previous invocation of the lib
 		if ((s_shm_sem = sem_open(get_shm_sem_name().c_str(), O_CREAT | O_EXCL, 0644, 1)) == SEM_FAILED)
@@ -293,8 +286,10 @@ void RdbInitializer::prepare4multitasking(size_t res_const_size, size_t res_var_
 		s_shm->total_mem_usage = 0;
 		s_shm->num_kids_running = num_planned_kids;
 		s_shm->num_kids_suspended = 0;
+		s_shm->untouchable_kid_idx = 0;
 
 		for (int i = 0; i < MAX_KIDS; ++i) {
+			s_shm->is_alive[i] = true;
 			s_shm->mem_usage[i] = 0;
 			s_shm->kid_progress[i] = 0;
 			s_shm->kid_res_offset[i] = 0;
@@ -308,7 +303,16 @@ pid_t RdbInitializer::launch_process()
 	if (s_shm_sem == SEM_FAILED || s_alloc_suspend_sem == SEM_FAILED || s_shm == (Shm *)MAP_FAILED)
 		verror("Not prepared for multitasking");
 
-	SigchldBlocker sb;
+	if (s_kid_index >= MAX_KIDS) 
+		verror("Too many child processes");
+
+	check_interrupt();
+
+	{
+		SemLocker sl(s_shm_sem);
+		if (s_shm->error_msg[0])
+			verror("%s", s_shm->error_msg);
+	}
 
 	pid_t pid = fork(); 
 
@@ -322,134 +326,192 @@ pid_t RdbInitializer::launch_process()
 		s_is_kid = true;
 
 		sigaction(SIGINT, &s_old_sigint_act, NULL);
-		sigaction(SIGCHLD, &s_old_sigchld_act, NULL);
-
+		sigaction(SIGCHLD, &s_old_sigchld_act, NULL);		
+		
 		SEXP r_multitasking_stdout = GetOption(install("gmultitasking_stdout"), R_NilValue);
 
-		if (!isLogical(r_multitasking_stdout) || !(int)LOGICAL(r_multitasking_stdout)[0])
-			fclose(stdout);
-		fclose(stderr);
-		fclose(stdin);
+		int devnull;
 
-		s_shm->mem_usage[s_kid_index] = s_init_mem_usage = get_unique_mem_usage(getpid());
-		s_shm->total_mem_usage += s_init_mem_usage;
-		check_interrupt();
+		if ((devnull = open("/dev/null", O_RDWR)) == -1){
+            verror("Failed to open /dev/null");
+        }
+
+        if (!isLogical(r_multitasking_stdout) || !(int)LOGICAL(r_multitasking_stdout)[0]) {
+            dup2(devnull, STDOUT_FILENO);
+        }
+
+        dup2(devnull, STDIN_FILENO);
+        dup2(devnull, STDERR_FILENO);
+        close(devnull);
+
+		int64_t delta_mem_usage = get_unique_mem_usage(getpid()) - s_shm->mem_usage[s_kid_index];
+		s_shm->mem_usage[s_kid_index] += delta_mem_usage;
+		s_shm->total_mem_usage += delta_mem_usage;
+
+		// set delay for child execution to prevent high initial bulk allocations
+		struct timespec req;
+		rdb::set_rel_timeout(LAUNCH_DELAY, req);
+		for (int i = 0; i < s_kid_index; ++i) {
+			nanosleep(&req, NULL);
+			check_interrupt();
+		}
 	}
 
 	return pid;
 }
 
+void RdbInitializer::check_kids_state(bool ignore_errors)
+{
+    int status;
+    pid_t pid;
+
+    while ((pid = waitpid((pid_t)-1, &status, WNOHANG)) > 0) {
+        int kid_idx = -1;
+
+        for (vector<LiveStat>::iterator ipid = s_running_pids.begin(); ipid != s_running_pids.end(); ++ipid) {
+            if (ipid->pid == pid) {
+                kid_idx = ipid->index;
+                swap(*ipid, s_running_pids.back());
+                s_running_pids.pop_back();
+
+                if (!ignore_errors && !WIFEXITED(status) && WIFSIGNALED(status) && WTERMSIG(status) != MISHA_EXIT_SIG){
+                    verror("Child process %d ended unexpectedly", (int)ipid->pid);
+				}
+
+                // choose a new untouchable kid: the one with maximal memory consumption
+                if (kid_idx == s_shm->untouchable_kid_idx && s_running_pids.size()) {
+                    int untouchable_kid_idx = s_running_pids.begin()->index;
+                    int64_t max_kid_mem_usage = s_shm->mem_usage[untouchable_kid_idx];
+                    for (vector<LiveStat>::iterator ipid2 = s_running_pids.begin() + 1; ipid2 < s_running_pids.end(); ++ipid2) {
+                        if (max_kid_mem_usage < s_shm->mem_usage[ipid2->index]) {
+                            untouchable_kid_idx = ipid->index;
+                            max_kid_mem_usage = s_shm->mem_usage[untouchable_kid_idx];
+                        }
+                    }
+                    s_shm->untouchable_kid_idx = untouchable_kid_idx;
+                }
+                s_shm->num_kids_running--;
+                break;
+            }
+        }
+
+        if (kid_idx >= 0)
+            s_shm->is_alive[kid_idx] = false;
+    }
+}
+
 void RdbInitializer::wait_for_kids(rdb::IntervUtils &iu)
 {
-	const int REPORT_INTERVAL_SECS = 3;
+	int64_t delay_msec = LAUNCH_DELAY / 2;
+    // bool slept_once = false;
 
-	struct timespec reg;
+	struct timespec timeout, last_progress_time, last_delay_change_time;
 	int last_progress = -1;
 
-	clock_gettime(CLOCK_REALTIME, &reg);
-	reg.tv_sec += REPORT_INTERVAL_SECS;
+	clock_gettime(CLOCK_REALTIME, &timeout);
+	last_progress_time = last_delay_change_time = timeout;
 
-	while (1) {
-		// sem_wait is interrupted by signal handlers
-		while (sem_timedwait(&s_signal_sem, &reg) < 0 && errno == EINTR)
-			;
+    while (1) {
+        clock_gettime(CLOCK_REALTIME, &timeout);
+        set_rel_timeout(delay_msec, timeout);
 
-		bool timeout = errno == ETIMEDOUT;
+        check_interrupt();
+        check_kids_state(false);
 
-		check_interrupt();
+        {
+            SemLocker sl(s_shm_sem);
+            if (s_shm->error_msg[0])
+                verror("%s", s_shm->error_msg);
+        }
 
-		{
-			SemLocker sl(s_shm_sem);
-			if (s_shm->error_msg[0])
-				verror("%s", s_shm->error_msg);
-		}
+        if (s_res_var_size) {
+            // update of data_size is atomic => don't use a semaphore
+            uint64_t res_num_records = 0;
 
-		SigchldBlocker sb;
-		while (s_dead_pids.size()) {
-			const DeathStat &death_stat = s_dead_pids.back();
+            for (int i = 0; i < get_num_kids(); ++i)
+                res_num_records += s_shm->kid_res_num_records[i];
+            iu.verify_max_data_size(res_num_records, "Result");
+        }
 
-//vdebug_print("%*s%d: SIGCHLD on %d, recovering: %ld, new total: %ld, running: %ld, suspended: %ld\n", s_kid_index + 1, "", (int)getpid(), (int)death_stat.pid,
-//s_shm->mem_usage[death_stat.index], s_shm->total_mem_usage - s_shm->mem_usage[death_stat.index], s_shm->num_kids_running, s_shm->num_kids_suspended);
-			{
-				SemLocker sl(s_shm_sem);
-				s_shm->num_kids_running--;
-				if (death_stat.index >= 0) {
-					s_shm->total_mem_usage -= s_shm->mem_usage[death_stat.index];
-					s_shm->mem_usage[death_stat.index] = 0;
-				}
+        if (s_running_pids.empty()) 
+            break;
 
-			}
-			if (WIFEXITED(death_stat.status))
-				s_dead_pids.pop_back();
-			else
-				verror("Child process %d ended unexpectedly", (int)death_stat.pid);
-		}
+        nanosleep(&timeout, NULL);
 
-		if (s_res_var_size) {
-			// update of data_size is atomic => don't use a semaphore
-			size_t res_num_records = 0;
+        int64_t total_mem_usage = s_shm->total_mem_usage;
 
-			for (int i = 0; i < get_num_kids(); ++i)
-				res_num_records += s_shm->kid_res_num_records[i];
-			iu.verify_max_data_size(res_num_records, "Result");
-		}
+        update_kids_mem_usage();
 
-		if (s_running_pids.empty()) 
-			break;
+        // Check the memory consumption increase from the last measurement. If the memory limit is going to be
+        // breached till the next measurement => increase the monitoring rate.
+        int64_t delta_mem_usage = s_shm->total_mem_usage - total_mem_usage;
+        int64_t time2reach_limit = -1;
 
-		if (timeout) {
-			// Update the memory usage of the kids with precise number.
-			// Usually the kids maintain their own usage based on their own rough estimation.
-			size_t total_mem_usage = 0;
+        if (delta_mem_usage > 0 && (uint64_t)s_shm->total_mem_usage <= (uint64_t)s_max_mem_usage)
+            time2reach_limit = delay_msec * ((s_max_mem_usage - s_shm->total_mem_usage) / delta_mem_usage);
 
-			for (vector<LiveStat>::const_iterator ipid = s_running_pids.begin(); ipid != s_running_pids.end(); ++ipid) {
-				size_t mem_usage = get_unique_mem_usage(ipid->pid);
+        if (time2reach_limit >= 0 && time2reach_limit < delay_msec) {
+            delay_msec = min(MEM_SYNC_DELAY / 2, delay_msec);
+            delay_msec = max(delay_msec, time2reach_limit);
+            clock_gettime(CLOCK_REALTIME, &last_delay_change_time);
+        } else if (delay_msec < REPORT_INTERVAL_DELAY && is_time_elapsed(2 * s_running_pids.size() * delay_msec, last_delay_change_time)) {
+            // after (2 * s_running_pids.size() * initial_delay) increase the delay twice, i.e. lower down the frequency update_kids_mem_usage() is called
+            // anyway do not drop the delay below what is required for progress reporting
+            delay_msec = min(2 * delay_msec, REPORT_INTERVAL_DELAY);
+            clock_gettime(CLOCK_REALTIME, &last_delay_change_time);
+        }
 
-				if (mem_usage) {
-					s_shm->mem_usage[ipid->index] = mem_usage;
-//vdebug_print("UPDATE MEM USAGE OF %d to %ld\n", ipid->index, s_shm->mem_usage[ipid->index]);
-					total_mem_usage += mem_usage;
-				}
-			}
+        if (is_time_elapsed(REPORT_INTERVAL_DELAY, last_progress_time)) {
+            // update of progress is atomic => don't use a semaphore
+            int progress = 0;
 
-			if (total_mem_usage) 
-//{
-				s_shm->total_mem_usage = total_mem_usage;
-//vdebug_print("UPDATE TOT USAGE to %ld, running: %ld, suspended: %ld, alive: %ld\n", s_shm->total_mem_usage, s_shm->num_kids_running, s_shm->num_kids_suspended,
-//s_running_pids.size());
-//}
+            for (int i = 0; i < get_num_kids(); ++i) 
+                progress += s_shm->kid_progress[i];
 
-			// update of progress is atomic => don't use a semaphore
-			int progress = 0;
+            progress = progress / get_num_kids();
+            if (progress < 100 && progress != last_progress)
+                REprintf("%d%%...", progress); 
+            else {
+                if (last_progress == -1) 
+                    REprintf("0%%...");
+                else
+                    REprintf(".");
+            }
+            last_progress = progress;
+            clock_gettime(CLOCK_REALTIME, &last_progress_time);
+        }
 
-			for (int i = 0; i < get_num_kids(); ++i) 
-				progress += s_shm->kid_progress[i];
-			
-			progress = progress / get_num_kids();
-			if (progress < 100 && progress != last_progress)
-				Rprintf("%d%%...", progress); 
-			else {
-				if (last_progress == -1) 
-					Rprintf("0%%...");
-				else
-					Rprintf(".");
-			}
-			last_progress = progress;
-			clock_gettime(CLOCK_REALTIME, &reg);
-			reg.tv_sec += REPORT_INTERVAL_SECS;
-		}
+        if (!s_shm->num_kids_running || (uint64_t)s_shm->total_mem_usage < (uint64_t)s_max_mem_usage) {
+            // wake up suspended processes
+            for (uint64_t i = 0; i < s_shm->num_kids_suspended; ++i) 
+                sem_post(s_alloc_suspend_sem);
+        }
+    }
 
+	if (last_progress >= 0) 
+		REprintf("100%%\n");
+}
 
-		if (!s_shm->num_kids_running || s_shm->total_mem_usage < s_max_mem_usage) {
-			// wake up suspended processes
-//vdebug_print("3 WAKING UP %d (timeout? %d)\n", (int)s_shm->num_kids_suspended, (int)timeout);
-			for (int i = 0; i < s_shm->num_kids_suspended; ++i) 
-				sem_post(s_alloc_suspend_sem);
+int64_t RdbInitializer::update_kids_mem_usage()
+{
+	// Update the memory usage of the kids with precise number.
+	// Usually the kids maintain their own usage based on their own rough estimation.
+	int64_t total_mem_usage = 0;
+
+	for (vector<LiveStat>::const_iterator ipid = s_running_pids.begin(); ipid != s_running_pids.end(); ++ipid) {
+		uint64_t mem_usage = get_unique_mem_usage(ipid->pid);
+
+		if (mem_usage) {
+			s_shm->mem_usage[ipid->index] = mem_usage;
+//vdebug_print("\t\tUPDATE MEM USAGE OF %d to %ld\n", ipid->index, s_shm->mem_usage[ipid->index]);
+			total_mem_usage += mem_usage;
 		}
 	}
 
-	if (last_progress >= 0) 
-		Rprintf("100%%\n");
+	if (total_mem_usage) 
+		s_shm->total_mem_usage = total_mem_usage;
+
+	return total_mem_usage;
 }
 
 void RdbInitializer::handle_error(const char *msg)
@@ -458,16 +520,16 @@ void RdbInitializer::handle_error(const char *msg)
 		{
 			SemLocker sl(s_shm_sem);
 			if (!s_shm->error_msg[0]) { // write an error message only if there were no error messages before
-				strncpy(s_shm->error_msg, msg, sizeof(s_shm->error_msg));
-				s_shm->error_msg[sizeof(s_shm->error_msg)] = '\0';
+				strncpy(s_shm->error_msg, msg, sizeof(s_shm->error_msg) - 1);
+				s_shm->error_msg[sizeof(s_shm->error_msg) - 1] = '\0';
 			}
 		}
-		exit(1);
+		rexit();
 	} else
-		error(msg);
+		errorcall(R_NilValue, msg);
 }
 
-void *RdbInitializer::allocate_res(size_t res_num_records)
+void *RdbInitializer::allocate_res(uint64_t res_num_records)
 {
 	if (!s_is_kid)
 		verror("allocate_res() cannot be called by parent process");
@@ -486,69 +548,97 @@ void *RdbInitializer::allocate_res(size_t res_num_records)
 	return &s_shm->res + s_shm->kid_res_offset[s_kid_index];
 }
 
-void RdbInitializer::out_of_memory()
-{
-	verror("Out of memory\n");
-}
-
 void RdbInitializer::sigint_handler(int)
 {
 	++s_sigint_fired;
 
 	// Normally this condition should be always true since the kid installs the default handler for SIGINT.
 	// However due to race condition the old handler might still be in use.
-	if (getpid() == s_parent_pid) {
-		// wake up myself from wait_for_kids()
-		sem_post(&s_signal_sem);
-		printf("CTL-C!\n");
-	}
+	if (getpid() == s_parent_pid)
+		REprintf("CTL-C!\n");
 }
 
 void RdbInitializer::sigchld_handler(int)
 {
-	// Normally this condition should be always true since the kid installs the default handler for SIGINT.
-	// However due to race condition the old handler might still be in use.
-	if (getpid() == s_parent_pid) {
-		int status;
-		pid_t pid;
-
-		while ((pid = waitpid((pid_t)-1, &status, WNOHANG)) > 0) {
-			int kid_idx = -1;
-
-			for (vector<LiveStat>::iterator ipid = s_running_pids.begin(); ipid != s_running_pids.end(); ++ipid) {
-				if (ipid->pid == pid) {
-					kid_idx = ipid->index;
-					if (ipid != s_running_pids.end() - 1)
-						*ipid = *(s_running_pids.end() - 1);
-					s_running_pids.pop_back();
-					break;
-				}
-			}
-
-			// malloc is forbidden in signal handlers, but the space was previously reserved for large number
-			// of PIDs, therefore push_back does not initiate malloc
-			s_dead_pids.push_back(DeathStat(pid, kid_idx, status));
-
-		}
-
-		sem_post(&s_signal_sem);
-	}
 }
 
 void RdbInitializer::get_open_fds(set<int> &fds)
 {
+#if defined(__APPLE__)
+    // This absolutely irrational code with all those funny reallocations and multiplication by 32 (haeh?) was inherited from here:
+    //      https://opensource.apple.com/source/Libc/Libc-825.26/darwin/proc_listpidspath.c.auto.html
+    // 
+    // It would be much more logical to call proc_pidinfo twice: the first time to get buf size and the second time to get the list
+    // of file descriptors. And indeed some internet sources advice to do that. However what is rational does not work and gives some
+    // 240 open file descriptors most of them are not vnodes at all. And even some of those who are vnodes, are complete junk. Smells
+    // like a memory leak. Probably the multiplication by 32 is really needed.
+    //
+    // All these problems come from the simple reason there is not manual for proc_pidinfo. Go figure why...
+	int	buf_used;
+    int fds_size = 0;
+    int num_fds;
+    unique_ptr<char[]> buf;
+
+	// get list of open file descriptors
+	buf_used = proc_pidinfo(getpid(), PROC_PIDLISTFDS, 0, NULL, 0);
+	if (buf_used <= 0)
+        return;
+
+	while (1) {
+		if (buf_used > fds_size) {
+			// if we need to allocate [more] space
+			while (buf_used > fds_size)
+				fds_size += (sizeof(struct proc_fdinfo) * 32);
+
+            buf = unique_ptr<char[]>(new char[fds_size]);
+		}
+
+		buf_used = proc_pidinfo(getpid(), PROC_PIDLISTFDS, 0, buf.get(), fds_size);
+		if (buf_used <= 0)
+            return;
+
+		if ((buf_used + sizeof(struct proc_fdinfo)) >= fds_size) {
+			// if not enough room in the buffer for an extra fd
+			buf_used = fds_size + sizeof(struct proc_fdinfo);
+			continue;
+		}
+
+		num_fds = buf_used / sizeof(struct proc_fdinfo);
+		break;
+	}
+
+    struct proc_fdinfo *fdinfo = (struct proc_fdinfo *)buf.get();
+    for (int i = 0; i < num_fds; ++i) {
+        if (fdinfo[i].proc_fdtype == PROX_FDTYPE_VNODE)
+            fds.insert(fdinfo[i].proc_fd);
+    }
+#else
+
+#ifdef __sun
+    #ifdef __XOPEN_OR_POSIX
+        #define _dirfd(dir) (dir->d_fd)
+    #else
+        #define _dirfd(dir) (dir->dd_fd)
+    #endif
+#else
+    #define _dirfd(dir) dirfd(dir)
+#endif
+
 	DIR *dir = opendir("/proc/self/fd");
 	struct dirent *dirp;
 
 	fds.clear();
-	while ((dirp = readdir(dir))) {
-		char *endptr;
-		int fd = strtol(dirp->d_name, &endptr, 10);
-		if (!*endptr && fd != dirfd(dir)) // name is a number (it can be also ".", "..", whatever...)
-			fds.insert(fd);
-	}
+    if (dir) {
+    	while ((dirp = readdir(dir))) {
+    		char *endptr;
+    		int fd = strtol(dirp->d_name, &endptr, 10);
+    		if (!*endptr && fd != _dirfd(dir)) // name is a number (it can be also ".", "..", whatever...)
+    			fds.insert(fd);
+    	}
 
-	closedir(dir);
+        closedir(dir);
+    }
+#endif
 }
 
 void RdbInitializer::vdebug_print(const char *fmt, ...)
@@ -557,18 +647,18 @@ void RdbInitializer::vdebug_print(const char *fmt, ...)
 	char buf[1000];
 
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
 	SemLocker sl(s_shm_sem);
-	printf("%s", buf);
+	REprintf("%s", buf);
 }
 
-void rdb::check_interrupt() throw (TGLException)
+void rdb::check_interrupt()
 {
 	if (RdbInitializer::s_sigint_fired)
 		TGLError("Command interrupted!");
-	report_alloc(0);
+	monitor_memusage();
 }
 
 void rdb::rerror(const char *fmt, ...)
@@ -577,7 +667,7 @@ void rdb::rerror(const char *fmt, ...)
 	char buf[1000];
 
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
 	RdbInitializer::handle_error(buf);
@@ -589,7 +679,7 @@ void rdb::verror(const char *fmt, ...)
 	char buf[1000];
 
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
 	if (RdbInitializer::s_ref_count)
@@ -609,8 +699,8 @@ SEXP rdb::rprotect(SEXP &expr)
 
 void rdb::runprotect(int count)
 {
-	if (RdbInitializer::s_protect_counter < count)
-		error("Number of calls to unprotect exceeds the number of calls to protect\n");
+	if (RdbInitializer::s_protect_counter < (uint64_t)count)
+		errorcall(R_NilValue, "Number of calls to unprotect exceeds the number of calls to protect\n");
 	UNPROTECT(count);
 	RdbInitializer::s_protect_counter -= count;
 }
@@ -619,7 +709,7 @@ void rdb::runprotect(SEXP &expr)
 {
 	if (expr != R_NilValue) {
 		if (RdbInitializer::s_protect_counter < 1)
-			error("Number of calls to unprotect exceeds the number of calls to protect\n");
+			errorcall(R_NilValue, "Number of calls to unprotect exceeds the number of calls to protect\n");
 		UNPROTECT_PTR(expr);
 		expr = R_NilValue;
 		RdbInitializer::s_protect_counter--;
@@ -649,8 +739,18 @@ void rdb::get_chrom_files(const char *dirname, vector<string> &chrom_files)
 	struct dirent *dirp;
 
 	while ((dirp = readdir(dir))) {
-		if (dirp->d_type == DT_REG && !strncmp(dirp->d_name, CHROM_FILE_PREFIX, CHROM_FILE_PREFIX_LEN))
-			chrom_files.push_back(dirp->d_name);
+		if (!strncmp(dirp->d_name, CHROM_FILE_PREFIX, CHROM_FILE_PREFIX_LEN)) {
+            if (dirp->d_type == DT_REG)
+                chrom_files.push_back(dirp->d_name);
+            else if (dirp->d_type == DT_UNKNOWN) {
+                struct stat sbuf;
+                char filename[PATH_MAX];
+
+                snprintf(filename, sizeof(filename), "%s/%s", dirname, dirp->d_name);
+                if (!stat(filename, &sbuf) && S_ISREG(sbuf.st_mode))
+                    chrom_files.push_back(dirp->d_name);
+            }
+        }
 	}
 
 	closedir(dir);
@@ -659,7 +759,7 @@ void rdb::get_chrom_files(const char *dirname, vector<string> &chrom_files)
 const char *rdb::get_groot(SEXP envir)
 {
 	// no need to protect the returned value
-	SEXP groot = findVar(install("GROOT"), envir);
+	SEXP groot = findVar(install("GROOT"), findVar(install(".misha"), envir));
 
 	if (!isString(groot))
 		verror("GROOT variable does not exist");
@@ -670,7 +770,7 @@ const char *rdb::get_groot(SEXP envir)
 const char *rdb::get_gwd(SEXP envir)
 {
 	// no need to protect the returned value
-	SEXP gwd = findVar(install("GWD"), envir);
+	SEXP gwd = findVar(install("GWD"), findVar(install(".misha"), envir));
 
 	if (!isString(gwd))
 		verror("GWD variable does not exist");
@@ -681,7 +781,7 @@ const char *rdb::get_gwd(SEXP envir)
 const char *rdb::get_glib_dir(SEXP envir)
 {
 	// no need to protect the returned value
-	SEXP glibdir = findVar(install(".GLIBDIR"), envir);
+	SEXP glibdir = findVar(install(".GLIBDIR"), findVar(install(".misha"), envir));
 
 	if (!isString(glibdir))
 		verror(".GLIBDIR variable does not exist");
@@ -749,10 +849,11 @@ SEXP rdb::eval_in_R(SEXP parsed_command, SEXP envir)
 SEXP rdb::run_in_R(const char *command, SEXP envir)
 {
 	SEXP expr;
-	SEXP parsed_expr;
+	SEXP parsed_expr = R_NilValue;
+    SEXPCleaner parsed_expr_cleaner(parsed_expr);
 	ParseStatus status;
 
-	rprotect(expr = allocVector(STRSXP, 1));
+	rprotect(expr = RSaneAllocVector(STRSXP, 1));
 	SET_STRING_ELT(expr, 0, mkChar(command));
 	rprotect(parsed_expr = R_ParseVector(expr, -1, &status, R_NilValue));
 	if (status != PARSE_OK)
@@ -766,7 +867,6 @@ struct RSaneSerializeData {
 	FILE *fp;
 };
 
-
 static void RSaneSeserializeCallback(void *_data)
 {
 	RSaneSerializeData *data = (RSaneSerializeData *)_data;
@@ -843,13 +943,37 @@ SEXP rdb::RSaneUnserialize(const char *fname)
 	return retv;
 }
 
+struct RSaneAllocVectorData {
+    SEXPTYPE type;
+    R_xlen_t len;
+    SEXP     retv;
+};
+
+static void RSaneAllocVectorCallback(void *_data)
+{
+	RSaneAllocVectorData *data = (RSaneAllocVectorData *)_data;
+    data->retv = allocVector(data->type, data->len);
+}
+
+SEXP rdb::RSaneAllocVector(SEXPTYPE type, R_xlen_t len)
+{
+    RSaneAllocVectorData data;
+
+    data.type = type;
+    data.len = len;
+    Rboolean ok = R_ToplevelExec(RSaneAllocVectorCallback, &data);
+    if (!ok)
+        verror("Allocation failed");
+    return data.retv;
+}
+
 SEXP rdb::get_rvector_col(SEXP v, const char *colname, const char *varname, bool error_if_missing)
 {
 	SEXP colnames = getAttrib(v, R_NamesSymbol);
 
 	if (!isVector(v) ||
-		length(v) && (!isString(colnames) || length(colnames) != length(v)) ||
-		!length(v) && !isNull(colnames))
+		(length(v) && (!isString(colnames) || length(colnames) != length(v))) ||
+		(!length(v) && !isNull(colnames)))
 		verror("Invalid format of %s", varname);
 
 	int numcols = isNull(colnames) ? 0 : length(colnames);
